@@ -1179,6 +1179,7 @@ func (m *Manager) addMonitor(monitorInfo *MonitorInfo) error {
 		MmHeight:           monitorInfo.MmHeight,
 		Enabled:            monitorInfo.Enabled,
 		uuid:               monitorInfo.UUID,
+		edid:               monitorInfo.EDID,
 		uuidV0:             monitorInfo.UuidV0,
 		Manufacturer:       monitorInfo.Manufacturer,
 		Model:              monitorInfo.Model,
@@ -1194,6 +1195,7 @@ func (m *Manager) addMonitor(monitorInfo *MonitorInfo) error {
 	monitor.Y = monitorInfo.Y
 	monitor.Width = monitorInfo.Width
 	monitor.Height = monitorInfo.Height
+	monitor.Brightness = 1
 
 	monitor.Reflects = getReflects(monitorInfo.Rotations)
 	monitor.Rotations = getRotations(monitorInfo.Rotations)
@@ -1911,7 +1913,10 @@ func (m *Manager) applyChanges() error {
 
 	configs := m.getSuitableSysMonitorConfigs(m.DisplayMode, monitorsId, monitors)
 	for _, config := range configs {
-		monitor := monitors.GetByUuid(config.UUID)
+		monitor := monitors.GetByUuidAndName(config.UUID, config.Name)
+		if monitor == nil {
+			monitor = monitors.GetByUuid(config.UUID)
+		}
 		if monitor == nil {
 			continue
 		}
@@ -1984,7 +1989,10 @@ func (m *Manager) applySysMonitorConfigs(mode byte, monitorsId monitorsId, monit
 	var primaryMonitorID uint32
 	var enabledMonitors []*Monitor
 	for _, monitor := range monitorMap {
-		monitorCfg := configs.getByUuid(monitor.uuid)
+		monitorCfg := configs.getByUuidAndName(monitor.uuid, monitor.Name)
+		if monitorCfg == nil {
+			monitorCfg = configs.getByUuid(monitor.uuid)
+		}
 		if monitorCfg == nil {
 			logger.Debug("disable monitor", monitor)
 			monitor.Enabled = false
