@@ -31,6 +31,7 @@ type Monitor struct {
 	m       *Manager
 	service *dbusutil.Service
 	uuid    string // uuid v1
+	edid    []byte
 	uuidV0  string
 	PropsMu sync.RWMutex
 
@@ -131,6 +132,7 @@ func (m *Monitor) clone() *Monitor {
 		m:                  m.m,
 		service:            m.service,
 		uuid:               m.uuid,
+		edid:               m.edid,
 		uuidV0:             m.uuidV0,
 		ID:                 m.ID,
 		Name:               m.Name,
@@ -317,6 +319,11 @@ func (m *Monitor) SetPosition(X, y int16) *dbus.Error {
 		return nil
 	}
 
+	if _dpy.DisplayMode == DisplayModeMirror {
+		logger.Debug("refuse to set position, because invalid display mode")
+		return nil
+	}
+
 	m.PropsMu.Lock()
 	defer m.PropsMu.Unlock()
 
@@ -539,6 +546,15 @@ func (monitors Monitors) GetByUuid(uuid string) *Monitor {
 	return nil
 }
 
+func (monitors Monitors) GetByUuidAndName(uuid, name string) *Monitor {
+	for _, monitor := range monitors {
+		if monitor.Name == name && monitor.getUuids().Contains(uuid) {
+			return monitor
+		}
+	}
+	return nil
+}
+
 const fillModeKeyDelimiter = ":"
 
 func (m *Monitor) generateFillModeKey() string {
