@@ -38,8 +38,8 @@ import (
 	daemonApps "github.com/linuxdeepin/go-dbus-factory/com.deepin.daemon.apps"
 	systemPower "github.com/linuxdeepin/go-dbus-factory/com.deepin.system.power"
 	proxy "github.com/linuxdeepin/go-dbus-factory/com.deepin.system.proxy"
-	x "github.com/linuxdeepin/go-x11-client"
-	"github.com/linuxdeepin/startdde/swapsched"
+	configManager "github.com/linuxdeepin/go-dbus-factory/org.desktopspec.ConfigManager"
+	systemd1 "github.com/linuxdeepin/go-dbus-factory/org.freedesktop.systemd1"
 	"github.com/linuxdeepin/go-gir/gio-2.0"
 	"github.com/linuxdeepin/go-lib/appinfo"
 	"github.com/linuxdeepin/go-lib/appinfo/desktopappinfo"
@@ -48,6 +48,7 @@ import (
 	"github.com/linuxdeepin/go-lib/keyfile"
 	"github.com/linuxdeepin/go-lib/strv"
 	"github.com/linuxdeepin/go-lib/xdg/basedir"
+	x "github.com/linuxdeepin/go-x11-client"
 )
 
 //go:generate dbusutil-gen em -type StartManager,SessionManager,Inhibitor
@@ -68,18 +69,20 @@ const (
 	KeyXDeepinCreatedBy     = "X-Deepin-CreatedBy"
 	KeyXDeepinAppID         = "X-Deepin-AppID"
 
-	uiAppSchedHooksDir = "/usr/lib/UIAppSched.hooks"
-	launchedHookDir    = uiAppSchedHooksDir + "/launched"
-
 	cpuFreqAdjustFile   = "/usr/share/startdde/app_startup.conf"
 	performanceGovernor = "performance"
 
 	restartRateLimitSeconds = 60
+
+	dsettingsAppID                            = "org.deepin.startdde"
+	dsettingsStartManagerName                 = "org.deepin.startdde.StartManager"
+	dsettingsEnableSystemdApplicationUnitsKey = "enable-systemd-application-units"
 )
 
 type StartManager struct {
 	xConn               *x.Conn
 	service             *dbusutil.Service
+	sysSigLoop          *dbusutil.SignalLoop
 	userAutostartPath   string
 	delayHandler        *mapDelayHandler
 	daemonApps          daemonApps.Apps
@@ -93,12 +96,15 @@ type StartManager struct {
 	appsDisableScaling  strv.Strv
 	mu                  sync.Mutex
 	appProxy            proxy.App
-	launchedHooks       []string
 
 	NeededMemory     uint64
 	systemPower      systemPower.Power
 	cpuFreqAdjustMap map[string]int32
 
+	userSystemd systemd1.Manager
+
+	enableSystemdApplicationUnit bool
+
 	//nolint
 	signals *struct {
 		AutostartChanged struct {
@@ -171,16 +177,6 @@ func (m *StartManager) enableCpuFreqLock(desktopFile string) error {
 	return nil
 }
 
-func (m *StartManager) execLaunchedHooks(desktopFile, cGroupName string) {
-	for _, name := range m.launchedHooks {
-		p := filepath.Join(launchedHookDir, name)
-		err := exec.Command(p, desktopFile, cGroupName).Run()
-		if err != nil {
-			logger.Warning("run cmd failed:", err)
-		}
-	}
-}
-
 func newStartManager(xConn *x.Conn, service *dbusutil.Service) *StartManager {
 	m := &StartManager{
 		service: service,
@@ -191,6 +187,7 @@ func newStartManager(xConn *x.Conn, service *dbusutil.Service) *StartManager {
 	m.settings = gio.NewSettings(gSchemaLauncher)
 	m.appsUseProxy = m.settings.GetStrv(gKeyAppsUseProxy)
 	m.appsDisableScaling = m.settings.GetStrv(gKeyAppsDisableScaling)
+	m.userSystemd = systemd1.NewManager(service.Conn())
 
 	gsettings.ConnectChanged(gSchemaLauncher, "*", func(key string) {
 		switch key {
@@ -213,7 +210,6 @@ func newStartManager(xConn *x.Conn, service *dbusutil.Service) *StartManager {
 	logger.Debugf("startManager proxychain confFile %q, bin: %q", m.proxyChainsConfFile, m.proxyChainsBin)
 
 	m.restartTimeMap = make(map[string]time.Time)
-	m.launchedHooks = getLaunchedHooks(launchedHookDir)
 	m.delayHandler = newMapDelayHandler(100*time.Millisecond,
 		m.emitSignalAutostartChanged)
 	sysBus, err := dbus.SystemBus()
@@ -221,13 +217,55 @@ func newStartManager(xConn *x.Conn, service *dbusutil.Service) *StartManager {
 		logger.Warning(err)
 	}
 
+	m.sysSigLoop = dbusutil.NewSignalLoop(sysBus, 10)
+	m.sysSigLoop.Start()
+
 	m.daemonApps = daemonApps.NewApps(sysBus)
 	m.systemPower = systemPower.NewPower(sysBus)
 	m.appProxy = proxy.NewApp(sysBus)
 	m.cpuFreqAdjustMap = m.getCpuFreqAdjustMap(cpuFreqAdjustFile)
+	m.initDSettings(sysBus)
 	return m
 }
 
+func (m *StartManager) initDSettings(sysBus *dbus.Conn) {
+	ds := configManager.NewConfigManager(sysBus)
+
+	startManagerPath, err := ds.AcquireManager(0, dsettingsAppID, dsettingsStartManagerName, "")
+	if err != nil {
+		logger.Warning(err)
+		return
+	}
+
+	dsStartManager, err := configManager.NewManager(sysBus, startManagerPath)
+	if err != nil {
+		logger.Warning(err)
+		return
+	}
+
+	getEnableSystemdApplicationUnit := func() {
+		v, err := dsStartManager.Value(0, dsettingsEnableSystemdApplicationUnitsKey)
+		if err != nil {
+			logger.Warning(err)
+			return
+		}
+
+		m.enableSystemdApplicationUnit = v.Value().(bool)
+	}
+
+	getEnableSystemdApplicationUnit()
+
+	dsStartManager.InitSignalExt(m.sysSigLoop, true)
+	_, err = dsStartManager.ConnectValueChanged(func(key string) {
+		if key == dsettingsEnableSystemdApplicationUnitsKey {
+			getEnableSystemdApplicationUnit()
+		}
+	})
+	if err != nil {
+		logger.Warning(err)
+	}
+}
+
 var _startManager *StartManager
 
 func (m *StartManager) GetInterfaceName() string {
@@ -250,11 +288,7 @@ func (m *StartManager) setRestartTime(appInfo *desktopappinfo.DesktopAppInfo, t
 }
 
 func (m *StartManager) GetApps() (map[uint32]string, *dbus.Error) {
-	if swapSchedDispatcher == nil {
-		return nil, dbusutil.ToError(errors.New("swap-sched disabled"))
-	}
-
-	return swapSchedDispatcher.GetAppsSeqDescMap(), nil
+	return nil, dbusutil.ToError(errors.New("swap-sched disabled"))
 }
 
 // deprecated
@@ -435,22 +469,7 @@ func (m *StartManager) runCommandWithOptions(exe string, args []string,
 		return nil
 	}
 
-	var uiApp *swapsched.UIApp
-	if swapSchedDispatcher != nil {
-		desc := getCmdDesc(exe, args)
-		uiApp, err = swapSchedDispatcher.NewApp(desc, nil)
-		if err != nil {
-			logger.Warning("dispatcher.NewApp error:", err)
-		}
-	}
-
-	var cmd *exec.Cmd
-	if uiApp != nil {
-		args = append([]string{"-g", "memory:" + uiApp.GetCGroup(), exe}, args...)
-		cmd = exec.Command(globalCgExecBin, args...)
-	} else {
-		cmd = exec.Command(exe, args...)
-	}
+	cmd := exec.Command(exe, args...)
 
 	if dirVar, ok := options["dir"]; ok {
 		if dirStr, ok := dirVar.Value().(string); ok {
@@ -461,7 +480,7 @@ func (m *StartManager) runCommandWithOptions(exe string, args []string,
 	}
 
 	err = cmd.Start()
-	return m.waitCmd(nil, cmd, err, uiApp, _name)
+	return m.waitCmd(nil, cmd, err, _name)
 }
 
 func (m *StartManager) getAppIdByFilePath(file string) string {
@@ -469,6 +488,11 @@ func (m *StartManager) getAppIdByFilePath(file string) string {
 }
 
 func (m *StartManager) shouldUseProxy(id string) bool {
+	// TODO: add support for application proxy
+	if m.enableSystemdApplicationUnit {
+		return false
+	}
+
 	m.mu.Lock()
 	if !m.appsUseProxy.Contains(id) {
 		m.mu.Unlock()
@@ -500,46 +524,43 @@ type IStartCommand interface {
 	StartCommand(files []string, ctx *appinfo.AppLaunchContext) (*exec.Cmd, error)
 }
 
+func (m *StartManager) createSystemdUnitForPID(appID string, desktopFile string, pid uint) {
+	if appID == "" {
+		appID = strings.TrimSuffix(filepath.Base(desktopFile), ".desktop")
+	}
+
+	unitName := fmt.Sprintf("app-dde-%s-%d.scope", appID, pid)
+
+	properties := []systemd1.Property{
+		{
+			Name:  "Description",
+			Value: dbus.MakeVariant("Launched by DDE"),
+		},
+		{
+			Name:  "PIDs",
+			Value: dbus.MakeVariant([]uint{pid}),
+		},
+		{
+			Name:  "CollectMode",
+			Value: dbus.MakeVariant("inactive-or-failed"),
+		},
+	}
+
+	m.userSystemd.StartTransientUnit(0, unitName, "fail", properties, nil)
+}
+
 func (m *StartManager) launch(appInfo *desktopappinfo.DesktopAppInfo, timestamp uint32,
 	files []string, iStartCmd IStartCommand, cmdName string) error {
-	// maximum RAM unit is MB
-	maxRAM, _ := appInfo.GetUint64(desktopappinfo.MainSection, "X-Deepin-MaximumRAM")
-	// unit is MB/s
-	blkioReadMBPS, _ := appInfo.GetUint64(desktopappinfo.MainSection, "X-Deepin-BlkioReadMBPS")
-	blkioWriteMBPS, _ := appInfo.GetUint64(desktopappinfo.MainSection, "X-Deepin-BlkioWriteMBPS")
-
 	desktopFile := appInfo.GetFileName()
 	logger.Debug("launch: desktopFile is", desktopFile)
 	var err error
 	var cmdPrefixes []string
-	var uiApp *swapsched.UIApp
 
 	err = m.enableCpuFreqLock(desktopFile)
 	if err != nil {
 		logger.Debug("cpu freq lock failed:", err)
 	}
 
-	if swapSchedDispatcher != nil {
-		if isDEComponent(appInfo) {
-			cmdPrefixes = []string{globalCgExecBin, "-g", "memory:" + swapSchedDispatcher.GetDECGroup()}
-		} else {
-			limit := &swapsched.AppResourcesLimit{
-				MemHardLimit:  maxRAM * 1e6,
-				BlkioReadBPS:  blkioReadMBPS * 1e6,
-				BlkioWriteBPS: blkioWriteMBPS * 1e6,
-			}
-			logger.Debugf("launch limit: %#v", limit)
-			uiApp, err = swapSchedDispatcher.NewApp(desktopFile, limit)
-			if err != nil {
-				logger.Warning("dispatcher.NewApp error:", err)
-			} else {
-				logger.Debug("launch: use cgexec")
-				cmdPrefixes = []string{globalCgExecBin, "-g",
-					"memory,freezer,blkio:" + uiApp.GetCGroup()}
-			}
-		}
-	}
-
 	appId := m.getAppIdByFilePath(desktopFile)
 	if appId != "" {
 		if m.shouldDisableScaling(appId) {
@@ -577,14 +598,11 @@ func (m *StartManager) launch(appInfo *desktopappinfo.DesktopAppInfo, timestamp
 
 	cmd, err := iStartCmd.StartCommand(files, ctx)
 
-	// exec launched hooks
-	cGroupName := ""
-	if uiApp != nil {
-		cGroupName = uiApp.GetCGroup()
+	if m.enableSystemdApplicationUnit {
+		m.createSystemdUnitForPID(appId, desktopFile, uint(cmd.Process.Pid))
 	}
-	go m.execLaunchedHooks(desktopFile, cGroupName)
 
-	return m.waitCmd(appInfo, cmd, err, uiApp, cmdName)
+	return m.waitCmd(appInfo, cmd, err, cmdName)
 }
 
 func newDesktopAppInfoFromFile(filename string) (*desktopappinfo.DesktopAppInfo, error) {
@@ -652,11 +670,7 @@ func (m *StartManager) launchAppActionAux(desktopFile, actionSection string, tim
 	return m.launch(appInfo, timestamp, nil, &targetAction, desktopFile+actionSection)
 }
 
-func (m *StartManager) waitCmd(appInfo *desktopappinfo.DesktopAppInfo, cmd *exec.Cmd, err error,
-	uiApp *swapsched.UIApp, cmdName string) error {
-	if uiApp != nil {
-		swapSchedDispatcher.AddApp(uiApp)
-	}
+func (m *StartManager) waitCmd(appInfo *desktopappinfo.DesktopAppInfo, cmd *exec.Cmd, err error, cmdName string) error {
 	if err != nil {
 		return err
 	}
@@ -704,18 +718,7 @@ func (m *StartManager) waitCmd(appInfo *desktopappinfo.DesktopAppInfo, cmd *exec
 				}
 			}
 		}
-		if uiApp != nil {
-			uiApp.SetStateEnd()
-		}
 	}()
-	if uiApp != nil {
-		go func() {
-			err := saveNeededMemory(cmdName, uiApp.GetCGroup())
-			if err != nil {
-				logger.Warning(err)
-			}
-		}()
-	}
 
 	return nil
 }
